/*
////////////////////////////////////////////////////////////////////////////////

Turns library created by ACWraith on 7/12/2004.

////////////////////////////////////////////////////////////////////////////////

LICENSE:

	Credit me with the creation of the library.

	You may use this in commercial or non-commercial products.

	You may create derivative works.

	You don't have to send me your changes.

////////////////////////////////////////////////////////////////////////////////
SUMMARY:

	This library manages the sorting and calling of turns.  Any datum, whether a
	mob or not, can be allowed to take turns in one or more TurnHandlers.  If
	desired Each datum can be allowed to take multiple turns per round.  If also
	desired, there is a hook for derivative types to sort a round's	order of
	turns differently.

	1)	You override the required procs for your whatever datum you want to take
	turns.  These procs are listed on the bottom of this file.

	2)	Then you add the datums to the TurnHandler and start/end turns at will.
	The handler will know who goes next and should not be bothered if a datum is
	removed.

////////////////////////////////////////////////////////////////////////////////
datum/TurnHandler

	*	You'll have to give the TurnHandler something to take turns.

		AddPlayer(datum/Player)
			Add a player to take turns in the TurnHandler and call Player's
			turns_AddedToHandler() trigger.  If this proc is called during a
			round, then the	player will not be told to take any turns until the
			next round.

		ClearPlayers()
			Remove all players and call their turns_RemovedFromHandler()
			triggers.  By default, this just calls RemovePlayer() iteratively.

		RemovePlayer(datum/Player)
			Remove Player from the handler and call Player's
			turns_RemovedFromHandler() trigger.  Note that a good derivative
			could make sure to automatically call the Player's
			turns_DoneWaitingForLastTurn() trigger if it was currently taking a
			turn.


	*	Naturally, you can pull out the player information stored.

		GetCurrentPlayer()
			Returns the player who is most recently taking a turn.  In a game
			with asynchronous turns, it would be the only person taking a turn.
			Between turns, it would be the last person to have taken a  turn.

		GetNextPlayerThisRound()
			Predict who the next player this round will be.  Internally, if
			null is returned, then the round is considered over.

		GetPlayers()
			Returns a list of all of the players.

		HasPlayer(datum/Player)
			Returns whether or not Player is being handled.

		NumPlayers()
			Returns the number of players being handled.



	*	These are probably the main procs that you will be calling in your game
	logic.

		StartTurn()
			Start the next turn, ending and starting rounds as necessary.
			You'll want to call at least the first turn manually when the game
			begins.  The next player will become the current player and it's
			turns_BeginTurn() trigger will be called.

		StopWaitingForTurn(datum/Player,Round,Turn,StartNext=FALSE)
			Call Player's turns_DoneWaitingForLastTurn() trigger if Player,
			Round and Turn are all the current values.  If StartNext, this
			immediately begin the next turn.



	*	The TurnHandler stores other information which could be usedful when
	determining time relative to the calls of datum triggers, but I find it's
	often not worth worrying about.

		GetCurrentRound()
			Get the current round identifier.

		GetCurrentTurn()
			Get the current turn identifier.



	*	The following procs are called internally if you wish to redefine them:

		RoundEnded()
			Handle whatever happens at the end of a round and tell the players
			the round ended.  By default, this just calls the turns_RoundEnded()
			triggers of the turn takers.

		RoundStarted()
			Handle whatever happens at the beginning of a round and tell the
			players the round started.  By default, this just calls the
			turns_RoundStarted() triggers of the turn takers.

		SortIntoTurnOrder(list/Players)
			Returns a list for the order of turns in a round.  You may wish to
			sort the list differently.  For instance, you could use some sort of
			initiative roll if mimicking a pen and paper RPG.

			This is called internally during RoundStarted().  By default, it
			simply returns the input.



////////////////////////////////////////////////////////////////////////////////



The following datum triggers are called (and probably should only be called) by
datum/TurnHandler.  It is your job to overwrite them for use in your game.



datum

	*	It is possible for turn takers to have more than one turn per round.
	You'll have to be able to tell the TurnHandler how many are left in a given
	round.  (Those using a variable amount of turns each round can rest easy
	knowing that the default TurnHandler only asks for the amount in the current
	round.)  The number of turns should be decreased after a turn is taken or
	else the round will never end.

		turns_TurnsLeftInRound(datum/TurnHandler/Handler,Round,Turn)
			Returns how many turns the datum has left at round Round and turn
			Turn.  Returns 0 by default so be sure to overwrite it for players.

	*	Normally, I define one of these to reset the amount of turns I have for
	the next round.

		turns_RoundEnded(datum/TurnHandler/Handler,Round,Turn)
			Handler finished round Round on turn Turn.

		turns_RoundStarted(datum/TurnHandler/Handler,Round,Turn)
			Handler is beginning round Round on turn Turn.



	*	This is where you'll probably want to put the meat of your code:

		turns_BeginTurn(datum/TurnHandler/Handler,Round,Turn)
			Handler has told the datum to begin turn Turn of round Round.  This
			would be a good time to give the turn taker access to whatever
			actions it can take.  During this proc, I also often lower the
			amount of turns the datum has left in the round.

		turns_DoneWaitingForLastTurn(datum/TurnHandler/Handler,Round,Turn)
			Handler will no longer wait for the datum to finish turn Turn of
			round Round.  For asynchronous turns, this would be a good time to
			remove the turn taker's ability to perform actions (providing one
			recognizes that turns_RemovedFromHandler() might need to perform
			this cleanup as well).  Otherwise, turn takers can continue
			performing while others are told to start their turn.  The
			TurnHandler doesn't care.


	*	I normally don't bother defining the following two procs in a session
	-based game.  However, a game with a persistent world could use them to
	prepare the turn taker.  For instance, players in an RPG could be warped on
	or off a battle screen.

		turns_AddedToHandler(datum/TurnHandler/Handler,Round,Turn)
			The datum has been added to handler Handler at turn Turn of round
			Round.

		turns_RemovedFromHandler(datum/TurnHandler/Handler,Round,Turn)
			The datum is no longer involved with Handler as of turn Turn of round
			Round.  You may wish to end any of the datum's ongoing turns here.
*/