//////////////////////////////////////////////////////////////////////////////////////////////////
datum/TurnHandler

	//============================================================================================
	var
		tmp
			currentRound	= 0
			currentTurn		= 0
			datum
				currentPlayer
				previousPlayer	// Used only if current was removed.
			list
				gamePlayers
				roundPlayers

			isEndOfRound	= FALSE
			isStartOfRound	= FALSE

	//============================================================================================
	//--------------------------------------------------------------------------------------------
	New()
		..()
		src.gamePlayers		= list()
		src.roundPlayers	= list()

	//============================================================================================
	proc
		//----------------------------------------------------------------------------------------
		//	Get the current round identifier.
		GetCurrentRound()
			return src.currentRound

		//----------------------------------------------------------------------------------------
		//	Get the current turn identifier.
		GetCurrentTurn()
			return src.currentTurn

		//----------------------------------------------------------------------------------------
		// Predict who the next player this round will be.
		// If null is returned, the round is over.
		GetNextPlayerThisRound()
			if ( isnull(src.roundPlayers) || (length(src.roundPlayers) <= 0) )
				return null

			var/currentPosition = src.roundPlayers.Find(src.GetCurrentPlayer())

			// Not in a round?
			if (currentPosition <= 0)
				return null

			var/datum/next

			// People may have multiple turns per round so loop back.
			if ( currentPosition >= length(src.roundPlayers) )
				currentPosition	= 0

				for(next in src.roundPlayers)
					if ( next.turns_TurnsLeftInRound(src,src.currentRound,src.currentTurn) > 0)
						return next

				// Not in a round.
				return null

			// Look at next in line.
			for(var/nextPos = (currentPosition+1) to length(src.roundPlayers) )
				next	= src.roundPlayers[nextPos]
				if ( next.turns_TurnsLeftInRound(src,src.currentRound,src.currentTurn) > 0)
					return next

			// Loop back to before in line since people may have multiple turns.
			for(var/nextPos = 1 to currentPosition )
				next	= src.roundPlayers[nextPos]
				if ( next.turns_TurnsLeftInRound(src,src.currentRound,src.currentTurn) > 0)
					return next

			// Not in a round.
			return null

		//----------------------------------------------------------------------------------------
		// Handle whatever happens at the end of a round and tell the players the round ended.
		RoundEnded()
			// Check for timing and bad calls.
			if (!src.isEndOfRound)
				return FALSE

			src.roundPlayers.Cut()

			// Notify all players of the end of an old round.
			for(var/datum/p in src.gamePlayers)
				p.turns_RoundEnded(src,src.currentRound,src.currentTurn)

			return TRUE

		//----------------------------------------------------------------------------------------
		// Handle whatever happens at the beginning of a round and tell the players the round started.
		RoundStarted()
			// Check for timing and bad calls.
			if (!src.isStartOfRound)
				return FALSE

			src.currentTurn	= 0

			var/datum/P

			// Notify all players of the start of a new round.
			for(P in src.gamePlayers)
				P.turns_RoundStarted(src,src.currentRound,src.currentTurn)

			// Allow derivatives to sort players however they want.
			src.roundPlayers	= src.SortIntoTurnOrder( src.GetPlayers() )

			// Make sure derivative did not return garbage.
			if ( isnull(src.roundPlayers) || !istype(src.roundPlayers,/list) )
				src.roundPlayers	= list()
			else
				for(P in src.roundPlayers)
					if ( !(P in src.gamePlayers) )
						src.roundPlayers	-= P

			return TRUE

		//----------------------------------------------------------------------------------------
		// Start the next turn, ending and starting rounds as necessary.
		StartTurn()
			// Fix holes in player list.
			if ( !isnull(src.previousPlayer) )
				src.currentPlayer	= src.previousPlayer
				src.previousPlayer	= null

			// Stop processing?
			if (src.NumPlayers() <= 0)
				src.currentPlayer	= null
				src.currentRound	= 0
				src.currentTurn		= 0
				return FALSE

			// Find next player.
			src.currentPlayer	= src.GetNextPlayerThisRound()

			// If next player in the round was not found, then try another round.
			var/endOnNextLoop	= FALSE

			while ( isnull(src.currentPlayer) )
				// End a round?
				if (endOnNextLoop)
					src.isEndOfRound	= TRUE
					src.RoundEnded(src.currentRound,src.currentTurn)
					src.isEndOfRound	= FALSE

				// Stop processing?
				if (src.NumPlayers() <= 0)
					src.currentRound	= 0
					src.currentTurn		= 0
					return FALSE

				// Start another round.
				src.currentRound++

				src.isStartOfRound	= TRUE
				src.RoundStarted(src.currentRound,src.currentTurn)
				src.isStartOfRound	= FALSE

				// Remember to end the round next time through.
				endOnNextLoop	= TRUE

				// Stop processing?
				if (src.NumPlayers() <= 0)
					src.currentRound	= 0
					src.currentTurn		= 0
					return FALSE

				// There are players so take the first.
				src.currentPlayer	= src.roundPlayers[1]

			// Start next turn.
			if (src.NumPlayers() > 0)
				src.currentTurn++
				src.currentPlayer.turns_BeginTurn(src,src.currentRound,src.currentTurn)
				return TRUE

			return FALSE

		//----------------------------------------------------------------------------------------
		// Stop waiting for turn Turn of round Round.
		// If StartNext, immediately begins the next turn.
		StopWaitingForTurn(datum/Player,Round,Turn,StartNext=FALSE)
			if ( (Player != src.currentPlayer) \
			|| (Round != src.currentRound) || (Turn != src.currentTurn) )
				return FALSE

			// End current player's turn.
			if (src.currentPlayer in src.roundPlayers)
				src.currentPlayer.turns_DoneWaitingForLastTurn(src,src.currentRound,src.currentTurn)

			// The round is over if there is no next player.
			if ( isnull(src.GetNextPlayerThisRound()) )
				src.isEndOfRound	= TRUE
				src.RoundEnded(src.currentRound,src.currentTurn)
				src.isEndOfRound	= FALSE

				// The handler is done if there are no more players.
				if (src.NumPlayers() <= 0)
					src.currentPlayer	= null
					src.currentRound	= 0
					src.currentTurn		= 0
					return TRUE

			if (StartNext)
				spawn(0)
					src.StartTurn()

			return TRUE

//////////////////////////////////////////////////////////////////////////////////////////////////