//////////////////////////////////////////////////////////////////////////////////////////////////
datum/TurnHandler

	//============================================================================================
	proc
		//----------------------------------------------------------------------------------------
		// Add a player to take turns.
		// If called during a round, the player will not take any turns until the next round.
		AddPlayer(datum/Player)
			if ( src.HasPlayer(Player) )
				return TRUE

			src.gamePlayers	+= Player
			Player.turns_AddedToHandler(src,src.currentRound,src.currentTurn)
			return TRUE

		//----------------------------------------------------------------------------------------
		// Remove all players.
		ClearPlayers()
			for(var/datum/P in src.gamePlayers)
				src.RemovePlayer(P)
			return TRUE

		//----------------------------------------------------------------------------------------
		// Returns the player who is most recently taking a turn.
		GetCurrentPlayer()
			if ( isnull(src.previousPlayer) )
				return src.currentPlayer
			return src.previousPlayer

		//----------------------------------------------------------------------------------------
		// Returns a list of all of the players.
		GetPlayers()
			return src.gamePlayers.Copy()

		//----------------------------------------------------------------------------------------
		// Returns whether or not Player is being handled.
		HasPlayer(datum/Player)
			return (Player in src.gamePlayers)

		//----------------------------------------------------------------------------------------
		// Returns the number of players being handled.
		NumPlayers()
			return length(src.gamePlayers)

		//----------------------------------------------------------------------------------------
		// Remove Player from the handler.
		RemovePlayer(datum/Player)
			if ( !src.HasPlayer(Player) )
				return TRUE

			if (Player in src.roundPlayers)

				// If the player is in the middle of a turn...
				if (Player == src.currentPlayer)
					// ...and there are more players...
					if ( length(src.roundPlayers) > 1 )
						// ...then mark the previous player to judge the next player from.
						var/currentPos	= src.roundPlayers.Find(src.currentPlayer)

						if ( currentPos == 1 )
							src.previousPlayer	= src.roundPlayers[length(src.roundPlayers)]
						else
							src.previousPlayer	= src.roundPlayers[currentPos - 1]

				src.roundPlayers	-= Player


			src.gamePlayers	-= Player
			Player.turns_RemovedFromHandler(src,src.currentRound,src.currentTurn)

			return TRUE

		//----------------------------------------------------------------------------------------
		// Returns a list for the order of turns in a round.
		// Called internally at the beginning of every round.
		// By default, returns the input, which, by default, is the order in which players joined.
		SortIntoTurnOrder(list/Players)
			return Players

//////////////////////////////////////////////////////////////////////////////////////////////////